/******************************************************************************
    Program ictdatedit, builds and edit database from analyze test files,
    for ic-tester program:
    - interface.c - source file for creating main window

    Copyright (C) 2001 Silvio Klaic, sklaic@student.math.hr
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
******************************************************************************/

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "support.h"
#include "ictdat.h"
#include "refresh.h"

GtkWidget* create_mainwin (void)
{
  GtkWidget *mainwin;
  GtkWidget *table1;
  GtkWidget *loadsatabase;
  GSList *type_group = NULL;
  GtkWidget *closebutton;
  GtkWidget *frame0;
  GtkWidget *frame1;
  GtkWidget *scrolledwindow1;
  GtkWidget *frame2;
  GtkWidget *scrolledwindow2;
  GtkWidget *frame3;
  GtkWidget *frame4;
  GtkWidget *frame5;
  GtkWidget *frame6;
  GtkWidget *frame7;
  GtkWidget *scrolledwindow3;

  mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (mainwin), "mainwin", mainwin);
  gtk_window_set_title (GTK_WINDOW (mainwin), g_strconcat("IC Tester database\
 editor v",VERSION,NULL));

  table1 = gtk_table_new (9, 5, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (mainwin), table1);

  databaseloc = gtk_label_new ("Database: none");
  gtk_widget_ref (databaseloc);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "databaseloc", databaseloc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (databaseloc);
  gtk_table_attach (GTK_TABLE (table1), databaseloc, 0, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (databaseloc, -2, 25);
  gtk_label_set_justify (GTK_LABEL (databaseloc), GTK_JUSTIFY_LEFT);

  savedatabase = gtk_button_new_with_label ("Save");
  gtk_widget_ref (savedatabase);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "savedatabase", savedatabase,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (savedatabase);
  gtk_table_attach (GTK_TABLE (table1), savedatabase, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (savedatabase, 80, 30);
  gtk_container_set_border_width (GTK_CONTAINER (savedatabase), 2);

  loadsatabase = gtk_button_new_with_label ("Load");
  gtk_widget_ref (loadsatabase);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "loadsatabase", loadsatabase,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (loadsatabase);
  gtk_table_attach (GTK_TABLE (table1), loadsatabase, 4, 5, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (loadsatabase, 80, 30);
  gtk_container_set_border_width (GTK_CONTAINER (loadsatabase), 2);

  typeIsize = gtk_label_new ("In database entry size: 0 Bytes");
  gtk_widget_ref (typeIsize);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "typeIsize", typeIsize,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (typeIsize);
  gtk_table_attach (GTK_TABLE (table1), typeIsize, 0, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (typeIsize, -2, 25);
  gtk_label_set_justify (GTK_LABEL (typeIsize), GTK_JUSTIFY_LEFT);

  islink = gtk_radio_button_new_with_label (type_group, "Link entry");
  type_group = gtk_radio_button_group (GTK_RADIO_BUTTON (islink));
  gtk_widget_ref (islink);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "islink", islink,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (islink);
  gtk_table_attach (GTK_TABLE (table1), islink, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (islink, 86, -2);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (islink), TRUE);

  istest = gtk_radio_button_new_with_label (type_group, "Test file entry");
  type_group = gtk_radio_button_group (GTK_RADIO_BUTTON (istest));
  gtk_widget_ref (istest);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "istest", istest,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (istest);
  gtk_table_attach (GTK_TABLE (table1), istest, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (istest, 86, -2);

  closebutton = gtk_button_new_with_label ("Close");
  gtk_widget_ref (closebutton);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "closebutton", closebutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (closebutton);
  gtk_table_attach (GTK_TABLE (table1), closebutton, 4, 5, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (closebutton, -2, 30);
  gtk_container_set_border_width (GTK_CONTAINER (closebutton), 2);


  savebutton = gtk_button_new_with_label ("Save");
  gtk_widget_ref (savebutton);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "savebutton", savebutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (savebutton);
  gtk_table_attach (GTK_TABLE (table1), savebutton, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (savebutton, -2, 30);
  gtk_container_set_border_width (GTK_CONTAINER (savebutton), 2);

  newbutton = gtk_button_new_with_label ("New");
  gtk_widget_ref (newbutton);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "newbutton", newbutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (newbutton);
  gtk_table_attach (GTK_TABLE (table1), newbutton, 3, 4, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (newbutton, -2, 30);
  gtk_container_set_border_width (GTK_CONTAINER (newbutton), 2);

  delbutton = gtk_button_new_with_label ("Delete");
  gtk_widget_ref (delbutton);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "delbutton", delbutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (delbutton);
  gtk_table_attach (GTK_TABLE (table1), delbutton, 4, 5, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (delbutton, -2, 30);
  gtk_container_set_border_width (GTK_CONTAINER (delbutton), 2);

  frame7 = gtk_frame_new ("Link location");
  gtk_widget_ref (frame7);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame7", frame7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame7);
  gtk_table_attach (GTK_TABLE (table1), frame7, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame7), 1);

  lfncombo = gtk_combo_new ();
  gtk_widget_ref (lfncombo);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "lfncombo", lfncombo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lfncombo);
  gtk_container_add (GTK_CONTAINER (frame7), lfncombo);
  gtk_widget_set_usize (lfncombo, 150, 25);

  inportbutton = gtk_button_new_with_label ("Import...");
  gtk_widget_ref (inportbutton);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "inportbutton", inportbutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (inportbutton);
  gtk_table_attach (GTK_TABLE (table1), inportbutton, 3, 4, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (inportbutton, -2, 30);
  gtk_container_set_border_width (GTK_CONTAINER (inportbutton), 2);

  exportbutton = gtk_button_new_with_label ("Export...");
  gtk_widget_ref (exportbutton);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "exportbutton", exportbutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (exportbutton);
  gtk_table_attach (GTK_TABLE (table1), exportbutton, 4, 5, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (exportbutton, -2, 30);
  gtk_container_set_border_width (GTK_CONTAINER (exportbutton), 2);

  frame0 = gtk_frame_new ("Entry name");
  gtk_widget_ref (frame0);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame0", frame0,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame0);
  gtk_table_attach (GTK_TABLE (table1), frame0, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame0), 1);


  tfncombo = gtk_combo_new ();
  gtk_widget_ref (tfncombo);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "tfncombo", tfncombo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tfncombo);
  gtk_container_add (GTK_CONTAINER (frame0), tfncombo);
  gtk_widget_set_usize (tfncombo, 150, 25);
  gtk_combo_set_use_arrows (GTK_COMBO(tfncombo), FALSE);

  frame1 = gtk_frame_new ("Tested ics");
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_table_attach (GTK_TABLE (table1), frame1, 0, 5, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 1);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (frame1), scrolledwindow1);
  gtk_widget_set_usize (scrolledwindow1, -2, 43);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  testediclist = gtk_text_new (NULL, NULL);
  gtk_widget_ref (testediclist);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "testediclist", testediclist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (testediclist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), testediclist);
  gtk_text_set_editable (GTK_TEXT (testediclist), TRUE);
  gtk_widget_realize (testediclist);

  frame2 = gtk_frame_new ("Authors");
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_table_attach (GTK_TABLE (table1), frame2, 0, 5, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 1);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow2);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "scrolledwindow2", scrolledwindow2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (frame2), scrolledwindow2);
  gtk_widget_set_usize (scrolledwindow2, -2, 43);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  authorslist = gtk_text_new (NULL, NULL);
  gtk_widget_ref (authorslist);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "authorslist", authorslist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (authorslist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), authorslist);
  gtk_text_set_editable (GTK_TEXT (authorslist), TRUE);
  gtk_widget_realize (authorslist);

  frame3 = gtk_frame_new ("IC Description");
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_table_attach (GTK_TABLE (table1), frame3, 0, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 1);

  description = gtk_entry_new_with_max_length (256);
  gtk_widget_ref (description);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "description", description,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (description);
  gtk_container_add (GTK_CONTAINER (frame3), description);
  gtk_widget_set_usize (description, 275, -2);

  frame4 = gtk_frame_new ("IC true name");
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_table_attach (GTK_TABLE (table1), frame4, 2, 5, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame4), 1);

  truename = gtk_entry_new_with_max_length (256);
  gtk_widget_ref (truename);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "truename", truename,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (truename);
  gtk_container_add (GTK_CONTAINER (frame4), truename);
  gtk_widget_set_usize (truename, 200, -2);

  frame5 = gtk_frame_new ("Modify date");
  gtk_widget_ref (frame5);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame5", frame5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame5);
  gtk_table_attach (GTK_TABLE (table1), frame5, 0, 2, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame5), 1);

  moddate = gtk_entry_new_with_max_length (20);
  gtk_widget_ref (moddate);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "moddate", moddate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (moddate);
  gtk_container_add (GTK_CONTAINER (frame5), moddate);
  gtk_entry_set_text (GTK_ENTRY (moddate), "YYYY/MM/DD-HH:MM-ZON");

  frame6 = gtk_frame_new ("Test data");
  gtk_widget_ref (frame6);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "frame6", frame6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame6);
  gtk_table_attach (GTK_TABLE (table1), frame6, 0, 5, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame6), 1);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow3);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "scrolledwindow3", scrolledwindow3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow3);
  gtk_container_add (GTK_CONTAINER (frame6), scrolledwindow3);
  gtk_widget_set_usize (scrolledwindow3, -2, 125);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  testtext = gtk_text_new (NULL, NULL);
  gtk_widget_ref (testtext);
  gtk_object_set_data_full (GTK_OBJECT (mainwin), "testtext", testtext,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (testtext);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), testtext);
  gtk_text_set_editable (GTK_TEXT (testtext), TRUE);
  gtk_widget_realize (testtext);

  gtk_signal_connect (GTK_OBJECT (mainwin), "destroy",
		      GTK_SIGNAL_FUNC (on_closebutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (mainwin), "delete_event",
		      GTK_SIGNAL_FUNC (on_closebutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (loadsatabase), "clicked",
                      GTK_SIGNAL_FUNC (on_loadsatabase_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (savedatabase), "clicked",
                      GTK_SIGNAL_FUNC (on_savedatabase_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (savebutton), "clicked",
                      GTK_SIGNAL_FUNC (on_savebutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (newbutton), "clicked",
                      GTK_SIGNAL_FUNC (on_newbutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (delbutton), "clicked",
                      GTK_SIGNAL_FUNC (on_delbutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (inportbutton), "clicked",
                      GTK_SIGNAL_FUNC (on_inportbutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (exportbutton), "clicked",
                      GTK_SIGNAL_FUNC (on_exportbutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (closebutton), "clicked",
                      GTK_SIGNAL_FUNC (on_closebutton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(tfncombo)->entry), "changed",
                      GTK_SIGNAL_FUNC (on_combotxt_changed), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(lfncombo)->entry), "changed",
                      GTK_SIGNAL_FUNC (on_comboltxt_changed), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(tfncombo)->popwin), "show",
                      GTK_SIGNAL_FUNC (on_tfncombo_selecting), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(tfncombo)->popwin), "hide",
                      GTK_SIGNAL_FUNC (on_tfncombo_selected), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(lfncombo)->popwin), "show",
                      GTK_SIGNAL_FUNC (on_lfncombo_selecting), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(lfncombo)->popwin), "hide",
                      GTK_SIGNAL_FUNC (on_lfncombo_selected), NULL);
  gtk_signal_connect (GTK_OBJECT (testtext), "changed",
                      GTK_SIGNAL_FUNC (on_testtext_changed), NULL);
  gtk_signal_connect (GTK_OBJECT (islink), "toggled",
		      GTK_SIGNAL_FUNC (on_islink_toggled), NULL);
  gtk_signal_connect (GTK_OBJECT (istest), "toggled",
		      GTK_SIGNAL_FUNC (on_istest_toggled), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(tfncombo)->entry),
		      "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  /* do update if edit one of those */
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(lfncombo)->entry),
	  	     "changed", GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(lfncombo)->entry),
		      "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  gtk_signal_connect (GTK_OBJECT (description), "changed",
	  GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (description), "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  gtk_signal_connect (GTK_OBJECT (truename), "changed", 
	            GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (truename), "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  gtk_signal_connect (GTK_OBJECT (testediclist), "changed", 
	            GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (testediclist), "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  gtk_signal_connect (GTK_OBJECT (authorslist), "changed", 
	            GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (authorslist), "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  gtk_signal_connect (GTK_OBJECT (moddate), "changed", 
	            GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (moddate), "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);
  gtk_signal_connect (GTK_OBJECT (testtext), "changed",
                    GTK_SIGNAL_FUNC (on_entry_check), NULL);
  gtk_signal_connect (GTK_OBJECT (testtext), "focus_out_event",
                      GTK_SIGNAL_FUNC (update_entries), NULL);

  return mainwin;
}/* end create_mainwin() */
